import errno
import logging
import sqlite3
from io import StringIO
from pathlib import Path

from argclass import Argument

from ..db import Interface
from .base import BaseParser


class SystemdNetworkdParser(BaseParser):
    output: Path = Argument(
        "--output", "-o", default=Path("/etc/systemd/network"), help="Output directory"
    )

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        output_path = self.output.resolve()
        logging.info("Generating systemd-networkd configuration to %s", output_path)

        for interface in Interface.list(conn):
            with StringIO() as f:
                f.write("[Match]\n")
                f.write(f"Name={interface.name}\n")
                f.write("\n")

                f.write("[Link]\n")
                f.write("ActivationPolicy=always-up\n")
                f.write("RequiredForOnline=no\n")
                f.write("\n")

                f.write("[Network]\n")
                for address in filter(None, [interface.ipv4, interface.ipv6]):
                    f.write(f"Address={address}\n")
                f.write("\n")
                network_content = f.getvalue()

            with StringIO() as f:
                f.write("[NetDev]\n")
                f.write("Kind=wireguard\n")
                f.write(f"Name={interface.name}\n")
                f.write(f"MTUBytes={interface.mtu}\n")
                f.write("\n")

                f.write("[WireGuard]\n")
                f.write(f"ListenPort={interface.listen_port}\n")
                f.write(f"PrivateKey={interface.private_key}\n")
                f.write("\n")

                for client in interface.clients(conn):
                    f.write(f"# Client: {client.alias}\n")
                    f.write("[WireGuardPeer]\n")
                    f.write(
                        "AllowedIPs={}\n".format(
                            ",".join(
                                map(str, filter(None, [client.ipv4, client.ipv6]))
                            ),
                        ),
                    )
                    f.write(f"PublicKey={client.public_key}\n")
                    if client.preshared_key:
                        f.write(f"PresharedKey={client.preshared_key}\n")
                    f.write(f"PersistentKeepalive={interface.persistent_keepalive}\n")
                    f.write("\n")

                netdev_content = f.getvalue()

            netdev_path = output_path / f"{interface.name}.netdev"
            network_path = output_path / f"{interface.name}.network"

            for path, content in [
                (netdev_path, netdev_content),
                (network_path, network_content),
            ]:
                path.parent.mkdir(parents=True, exist_ok=True)
                logging.info(
                    "Writing configuration for %s to: %s", interface.name, path
                )
                path.write_text(content)
                path.chmod(0o640)
        return 0


class WGQuickParser(BaseParser):
    output: Path = Argument(
        "--output", "-o", default=Path("/etc/wireguard"), help="Output directory"
    )

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        output_path = self.output.resolve()
        logging.info("Generating wg-quick configuration to %s", output_path)

        for interface in Interface.list(conn):
            with StringIO() as f:
                f.write("[Interface]\n")
                f.write(f"ListenPort={interface.listen_port}\n")
                f.write(f"PrivateKey={interface.private_key}\n")
                f.write(f"MTU={interface.mtu}\n")
                f.write(
                    "Address={}\n".format(
                        ",".join(
                            map(str, filter(None, [interface.ipv4, interface.ipv6]))
                        ),
                    ),
                )
                f.write("\n")

                for client in interface.clients(conn):
                    f.write(f"# Client: {client.alias}\n")
                    f.write("[Peer]\n")
                    f.write(
                        "AllowedIPs={}\n".format(
                            ",".join(
                                map(str, filter(None, [client.ipv4, client.ipv6]))
                            ),
                        ),
                    )
                    f.write(f"PublicKey={client.public_key}\n")
                    f.write(f"PersistentKeepalive={interface.persistent_keepalive}\n")
                    if client.preshared_key:
                        f.write(f"PresharedKey={client.preshared_key}\n")

                    f.write("\n")

                config_content = f.getvalue()

            config_path = output_path / f"{interface.name}.conf"
            config_path.parent.mkdir(parents=True, exist_ok=True)
            logging.info("Writing configuration to: %s", config_path)
            config_path.write_text(config_content)
            config_path.chmod(0o640)

        return 0


class RenderParser(BaseParser):
    systemd = SystemdNetworkdParser()
    wgquick = WGQuickParser()

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        self.print_help()
        return errno.EINVAL



from pathlib import Path

import argclass
from argclass import Argument

from .base import BaseParser
from .client import ClientCommands
from .interface import InterfaceCommands
from .render import RenderParser


class Parser(BaseParser):
    log_level = argclass.LogLevel
    db_path: Path = Argument(default=None, help="Path to the database file")
    output_format: str = Argument(
        "--output-format",
        "-f",
        default="table",
        choices=["table", "json", "csv", "tsv"],
        help="Output format",
    )

    interface: InterfaceCommands = InterfaceCommands()
    client: ClientCommands = ClientCommands()
    render: RenderParser = RenderParser(description="Render server config files")



import errno
import ipaddress
import logging
import sqlite3
from random import randint

from argclass import Argument, Nargs

from wg_gen.cli import BaseParser
from wg_gen.cli.client import ClientBaseParser
from wg_gen.db import Interface
from wg_gen.keygen import keygen
from wg_gen.table import SimpleTable


NON_LOCAL_NETS = frozenset(
    map(
        ipaddress.ip_network,
        [
            "1.0.0.0/8",
            "2.0.0.0/8",
            "3.0.0.0/8",
            "4.0.0.0/6",
            "8.0.0.0/7",
            "11.0.0.0/8",
            "12.0.0.0/6",
            "16.0.0.0/4",
            "32.0.0.0/3",
            "64.0.0.0/2",
            "128.0.0.0/3",
            "160.0.0.0/5",
            "168.0.0.0/6",
            "172.0.0.0/12",
            "172.32.0.0/11",
            "172.64.0.0/10",
            "172.128.0.0/9",
            "173.0.0.0/8",
            "174.0.0.0/7",
            "176.0.0.0/4",
            "192.0.0.0/9",
            "192.128.0.0/11",
            "192.160.0.0/13",
            "192.169.0.0/16",
            "192.170.0.0/15",
            "192.172.0.0/14",
            "192.176.0.0/12",
            "192.192.0.0/10",
            "193.0.0.0/8",
            "194.0.0.0/7",
            "196.0.0.0/6",
            "200.0.0.0/5",
            "208.0.0.0/4",
            "2000::/3",
        ],
    )
)


class InterfaceAddParser(BaseParser):
    """Add a new WireGuard interface"""

    name: str = Argument("name", help="Interface name (e.g. wg0)")
    ipv4: ipaddress.IPv4Interface | None = Argument(
        default=None,
        help="IPv4 interface for server, all subnet addresses wil be used for clients (e.g. 10.0.0.1/24)",
        type=ipaddress.IPv4Interface,
    )
    ipv6: ipaddress.IPv6Interface | None = Argument(
        default=None,
        help="IPv6 interface for server, all subnet addresses wil be used for clients (e.g. fd00::1/64)",
        type=ipaddress.IPv6Interface,
    )
    mtu: int = Argument(default=1420, help="MTU to use for the interface")
    listen_port: int = Argument(
        default=0,
        help="Server listen port, if not specified a random port between 1024 and 65000 will be used",
    )
    endpoint: str = Argument(
        required=True, help="Server endpoint hostname or IP for clients"
    )
    dns: list[ipaddress.IPv4Address | ipaddress.IPv6Address] = Argument(
        nargs=Nargs.ONE_OR_MORE,
        default=["1.1.1.1", "8.8.8.8"],
        help="DNS servers for clients",
        type=ipaddress.ip_address,
    )
    allowed_ips: list[str] = Argument(
        nargs=Nargs.ONE_OR_MORE,
        default=["0.0.0.0/0", "2000::/3"],
        help="Allowed IPs for peers. Special value 'non-local' can be used to set all non-local networks",
        type=str,
    )
    persistent_keepalive: int = Argument(
        default=15, help="Persistent keepalive seconds"
    )

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        private_key, public_key = keygen()
        allowed_ips: set[ipaddress.IPv4Network | ipaddress.IPv6Network] = set()

        for allowed_ip in self.allowed_ips:
            if allowed_ip == "non-local":
                allowed_ips.update(NON_LOCAL_NETS)
            else:
                allowed_ips.add(ipaddress.ip_network(allowed_ip))

        listen_port = self.listen_port if self.listen_port else randint(1024, 65000)

        interface = Interface(
            name=self.name,
            ipv4=self.ipv4,
            ipv6=self.ipv6,
            mtu=self.mtu,
            listen_port=listen_port,
            endpoint=self.endpoint,
            dns=self.dns,
            allowed_ips=sorted(allowed_ips, key=str),
            persistent_keepalive=self.persistent_keepalive,
            public_key=public_key,
            private_key=private_key,
        )
        try:
            interface.save(conn)
        except ValueError as e:
            logging.error("%s", e)
            return 1
        return 0


class InterfaceListParser(BaseParser):
    """List all interfaces"""

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        table = SimpleTable(
            "Interface",
            "Endpoint",
            "Public Key",
            "IPv4",
            "IPv6",
            "MTU",
            "Listen Port",
            "DNS",
            "Allowed IPs",
            "Address Shift",
            title="WireGuard Interfaces",
        )
        interfaces = Interface.list(conn)
        for interface in interfaces:
            table.add_row(
                interface.name,
                interface.endpoint,
                interface.public_key,
                str(interface.ipv4),
                str(interface.ipv6),
                str(interface.mtu),
                str(interface.listen_port),
                "\n".join(str(dns) for dns in interface.dns),
                "\n".join(str(allowed_ip) for allowed_ip in interface.allowed_ips),
                str(interface.address_shift),
            )

        table.print(self.__parent__.__parent__.output_format)  # type: ignore[union-attr]
        return 0


class InterfaceRemoveParser(ClientBaseParser):
    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        try:
            interface = Interface.load(conn, self.interface)
        except LookupError:
            logging.error("Interface %s was not found", self.interface)
            return 1

        logging.info("Removing interface %s", self.interface)
        interface.remove(conn)
        return 0


class InterfaceCommands(BaseParser):
    """Manage WireGuard interfaces"""

    add: InterfaceAddParser = InterfaceAddParser()
    remove: InterfaceRemoveParser = InterfaceRemoveParser()
    list: InterfaceListParser = InterfaceListParser()

    def __call__(self, *args, **kwargs):
        self.print_help()
        exit(errno.EINVAL)



import contextlib
import ipaddress
import sqlite3
from dataclasses import dataclass, field
from datetime import datetime
from pathlib import Path
from typing import Iterator

from .keygen import keygen, preshared_keygen


def init_db(conn: sqlite3.Connection):
    cur = conn.cursor()
    # interfaces table
    cur.execute(
        """
        CREATE TABLE IF NOT EXISTS interfaces (
            name TEXT PRIMARY KEY UNIQUE NOT NULL,
            created_at TEXT NOT NULL DEFAULT CURRENT_TIMESTAMP,
            ipv4 TEXT DEFAULT NULL,
            ipv6 TEXT DEFAULT NULL,
            address_shift INTEGER NOT NULL DEFAULT 1,
            private_key TEXT NOT NULL,
            public_key TEXT NOT NULL,
            mtu INTEGER NOT NULL,
            listen_port INTEGER,
            endpoint TEXT NOT NULL,
            dns TEXT NOT NULL,
            allowed_ips TEXT NOT NULL,
            persistent_keepalive INTEGER NOT NULL
        )
        """,
    )

    # clients table
    cur.execute(
        """
        CREATE TABLE IF NOT EXISTS clients (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            interface TEXT NOT NULL,
            alias TEXT NOT NULL,
            public_key TEXT NOT NULL,
            preshared_key TEXT DEFAULT NULL,
            ipv4 TEXT DEFAULT NULL,
            ipv6 TEXT DEFAULT NULL,
            created_at TEXT NOT NULL DEFAULT CURRENT_TIMESTAMP,
            FOREIGN KEY (interface) REFERENCES interfaces(name),
            UNIQUE (interface, alias)
        )""",
    )

    conn.commit()


@contextlib.contextmanager
def db_connection(db_path: Path):
    conn = sqlite3.connect(str(db_path))
    conn.row_factory = sqlite3.Row
    conn.execute("BEGIN IMMEDIATE TRANSACTION")
    try:
        yield conn
    except:
        conn.rollback()
        raise
    else:
        conn.commit()
    finally:
        conn.close()


@dataclass(frozen=False)
class Interface:
    name: str
    ipv4: ipaddress.IPv4Interface | None
    ipv6: ipaddress.IPv6Interface | None
    private_key: str
    public_key: str
    mtu: int
    listen_port: int
    endpoint: str
    dns: list[ipaddress.IPv4Address | ipaddress.IPv6Address]
    allowed_ips: list[ipaddress.IPv4Network | ipaddress.IPv6Network] = field(
        default_factory=list
    )
    address_shift: int = 1
    persistent_keepalive: int = 15
    created_at: datetime = field(default_factory=datetime.now)

    @classmethod
    def load(cls, conn: sqlite3.Connection, interface_name: str) -> "Interface":
        """Load an interface from the database"""
        cur = conn.cursor()
        cur.execute("SELECT * FROM interfaces WHERE name = ?", (interface_name,))
        result = cur.fetchone()
        if not result:
            raise LookupError("Interface not found")

        return cls(
            name=result["name"],
            created_at=datetime.strptime(result["created_at"], "%Y-%m-%d %H:%M:%S"),
            ipv4=ipaddress.IPv4Interface(result["ipv4"]) if result["ipv4"] else None,
            ipv6=ipaddress.IPv6Interface(result["ipv6"]) if result["ipv6"] else None,
            address_shift=result["address_shift"],
            private_key=result["private_key"],
            public_key=result["public_key"],
            mtu=result["mtu"],
            listen_port=result["listen_port"],
            endpoint=result["endpoint"],
            dns=list(map(ipaddress.ip_address, result["dns"].split(","))),
            allowed_ips=list(
                map(ipaddress.ip_network, result["allowed_ips"].split(","))
            ),
            persistent_keepalive=result["persistent_keepalive"],
        )

    def check_address_space(self) -> None:
        """Check that server address is in the network and there is room
        for at least one client address (server + shift=1)."""
        if self.ipv4:
            first_client_v4 = self.ipv4 + 1
            if first_client_v4.ip not in self.ipv4.network:
                raise ValueError(
                    f"IPv4 network {self.ipv4.network} has no room "
                    f"for client addresses (server address {self.ipv4.ip})",
                )
        if self.ipv6:
            first_client_v6 = self.ipv6 + 1
            if first_client_v6.ip not in self.ipv6.network:
                raise ValueError(
                    f"IPv6 network {self.ipv6.network} has no room "
                    f"for client addresses (server address {self.ipv6.ip})",
                )

    def check_ip_conflicts(self, conn: sqlite3.Connection) -> None:
        """Check that IPv4/IPv6 subnets don't overlap with other interfaces"""
        cur = conn.cursor()
        cur.execute(
            "SELECT name, ipv4, ipv6 FROM interfaces WHERE name != ?",
            (self.name,),
        )
        for row in cur.fetchall():
            if self.ipv4 and row["ipv4"]:
                existing_v4 = ipaddress.IPv4Interface(row["ipv4"])
                if self.ipv4.network.overlaps(existing_v4.network):
                    raise ValueError(
                        f"IPv4 subnet {self.ipv4.network} overlaps with "
                        f"interface '{row['name']}' ({existing_v4.network})",
                    )
            if self.ipv6 and row["ipv6"]:
                existing_v6 = ipaddress.IPv6Interface(row["ipv6"])
                if self.ipv6.network.overlaps(existing_v6.network):
                    raise ValueError(
                        f"IPv6 subnet {self.ipv6.network} overlaps with "
                        f"interface '{row['name']}' ({existing_v6.network})",
                    )

    def save(self, conn: sqlite3.Connection) -> None:
        """Save the interface to the database"""
        self.check_address_space()
        self.check_ip_conflicts(conn)
        cur = conn.cursor()
        cur.execute(
            """
            INSERT INTO interfaces(
                name,
                created_at,
                ipv4,
                ipv6,
                address_shift,
                private_key,
                public_key,
                mtu,
                listen_port,
                endpoint,
                dns,
                allowed_ips,
                persistent_keepalive
            )
            VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) ON CONFLICT DO UPDATE
            SET created_at = excluded.created_at,
                ipv4 = excluded.ipv4,
                ipv6 = excluded.ipv6,
                address_shift = excluded.address_shift,
                private_key = excluded.private_key,
                public_key = excluded.public_key,
                mtu = excluded.mtu,
                listen_port = excluded.listen_port,
                endpoint = excluded.endpoint,
                dns = excluded.dns,
                allowed_ips = excluded.allowed_ips,
                persistent_keepalive = excluded.persistent_keepalive
            """,
            (
                self.name,
                self.created_at.strftime("%Y-%m-%d %H:%M:%S"),
                str(self.ipv4) if self.ipv4 else None,
                str(self.ipv6) if self.ipv6 else None,
                self.address_shift,
                self.private_key,
                self.public_key,
                self.mtu,
                self.listen_port,
                self.endpoint,
                ",".join(map(str, self.dns)),
                ",".join(map(str, self.allowed_ips)),
                self.persistent_keepalive,
            ),
        )

    def generate_client_ipv4(self) -> ipaddress.IPv4Interface | None:
        if not self.ipv4:
            return None
        result = self.ipv4 + self.address_shift
        if result.ip not in self.ipv4.network:
            raise ValueError(
                f"IPv4 address pool exhausted for {self.ipv4.network}",
            )
        return result

    def generate_client_ipv6(self) -> ipaddress.IPv6Interface | None:
        if not self.ipv6:
            return None
        result = self.ipv6 + self.address_shift
        if result.ip not in self.ipv6.network:
            raise ValueError(
                f"IPv6 address pool exhausted for {self.ipv6.network}",
            )
        return result

    def create_client(
        self,
        conn: sqlite3.Connection,
        alias: str,
        preshared_key: bool = False,
    ) -> tuple["Client", str]:
        psk: str | None = preshared_keygen() if preshared_key else None

        ipv4_iface = self.generate_client_ipv4()
        ipv6_iface = self.generate_client_ipv6()
        ipv4 = ipv4_iface.ip if ipv4_iface else None
        ipv6 = ipv6_iface.ip if ipv6_iface else None

        self.address_shift += 1
        self.save(conn)

        private, public = keygen()

        client = Client(
            interface=self.name,
            alias=alias,
            public_key=public,
            preshared_key=psk,
            ipv4=ipv4,
            ipv6=ipv6,
        )
        client.save(conn)
        return client, private

    def clients(self, conn: sqlite3.Connection) -> Iterator["Client"]:
        cur = conn.cursor()
        cur.execute(
            "SELECT alias FROM clients WHERE interface = ? ORDER BY id", (self.name,)
        )
        for row in cur.fetchall():
            yield Client.load(conn, row["alias"], self.name)

    @classmethod
    def list(cls, conn: sqlite3.Connection) -> Iterator["Interface"]:
        cur = conn.cursor()
        cur.execute("SELECT name FROM interfaces ORDER BY name")
        for row in cur.fetchall():
            yield cls.load(conn, row["name"])

    def remove(self, conn: sqlite3.Connection) -> None:
        """Remove the interface from the database"""
        cur = conn.cursor()
        cur.execute(
            "DELETE FROM clients WHERE interface = ?",
            (self.name,),
        )
        cur.execute(
            "DELETE FROM interfaces WHERE name = ?",
            (self.name,),
        )


@dataclass(frozen=False)
class Client:
    interface: str
    alias: str
    public_key: str
    preshared_key: str | None
    ipv4: ipaddress.IPv4Address | None
    ipv6: ipaddress.IPv6Address | None
    created_at: datetime = field(default_factory=datetime.now)

    @classmethod
    def load(cls, conn: sqlite3.Connection, alias: str, interface: str) -> "Client":
        cur = conn.cursor()
        cur.execute(
            "SELECT * FROM clients WHERE alias = ? AND interface = ?",
            (alias, interface),
        )
        result = cur.fetchone()
        if not result:
            raise LookupError("Client not found")

        return cls(
            interface=result["interface"],
            alias=result["alias"],
            public_key=result["public_key"],
            preshared_key=result["preshared_key"],
            created_at=datetime.strptime(result["created_at"], "%Y-%m-%d %H:%M:%S"),
            ipv4=ipaddress.IPv4Address(result["ipv4"]) if result["ipv4"] else None,
            ipv6=ipaddress.IPv6Address(result["ipv6"]) if result["ipv6"] else None,
        )

    def save(self, conn: sqlite3.Connection) -> None:
        """Save the client to the database"""
        cur = conn.cursor()
        cur.execute(
            """
            INSERT INTO clients(
                interface,
                alias,
                public_key,
                preshared_key,
                ipv4,
                ipv6,
                created_at
            )
            VALUES (?, ?, ?, ?, ?, ?, ?) ON CONFLICT DO UPDATE
            SET created_at = excluded.created_at,
                public_key = excluded.public_key,
                preshared_key = excluded.preshared_key,
                ipv4 = excluded.ipv4,
                ipv6 = excluded.ipv6
            """,
            (
                self.interface,
                self.alias,
                self.public_key,
                self.preshared_key,
                str(self.ipv4) if self.ipv4 else None,
                str(self.ipv6) if self.ipv6 else None,
                self.created_at.strftime("%Y-%m-%d %H:%M:%S"),
            ),
        )

    def remove(self, conn: sqlite3.Connection) -> None:
        """Remove the client from the database"""
        cur = conn.cursor()
        cur.execute(
            "DELETE FROM clients WHERE interface = ? AND alias = ?",
            (self.interface, self.alias),
        )



import errno
import sqlite3

import argclass


class BaseParser(argclass.Parser):
    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        if self.current_subparser is not None:
            return self.current_subparser(conn)  # type: ignore[call-arg]
        self.print_help()
        exit(errno.EINVAL)



import configparser
import errno
import io
import logging
import sqlite3

import argclass

from argclass import Argument
from qrcode.main import QRCode  # type: ignore[import-untyped]
from rich import get_console
from rich.panel import Panel

from .base import BaseParser
from wg_gen.db import Client, Interface
from wg_gen.table import SimpleTable


class ClientBaseParser(BaseParser):
    """Base class for interface-related commands"""

    interface: str = Argument("interface", help="WireGuard interface name")


class ClientAddParser(ClientBaseParser):
    """Add a new client to an interface"""

    alias: str = Argument("alias", help="Client alias (unique per interface)")
    preshared_key: bool = False
    force: bool = False
    qr: bool = False

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        try:
            interface = Interface.load(conn, self.interface)
        except LookupError:
            print(f"Error: Interface '{self.interface}' not found")
            return 1

        try:
            Client.load(conn, self.alias, self.interface)
        except LookupError:
            pass
        else:
            if not self.force:
                logging.error(
                    "Error: Client '%s' already exists for interface '%s', use --force to overwrite",
                    self.alias,
                    self.interface,
                )
                return 1

        try:
            client, private_key = interface.create_client(
                conn,
                alias=self.alias,
                preshared_key=self.preshared_key,
            )
        except ValueError as e:
            logging.error("%s", e)
            return 1

        config = configparser.RawConfigParser()
        config.optionxform = str  # type: ignore[assignment]

        config.add_section("Interface")
        config.add_section("Peer")

        addresses = []
        if client.ipv4:
            addresses.append(str(client.ipv4))
        if client.ipv6:
            addresses.append(str(client.ipv6))

        config.set("Interface", "Address", ", ".join(addresses))
        config.set("Interface", "PrivateKey", private_key)
        config.set("Interface", "DNS", ",".join(map(str, interface.dns)))
        config.set("Interface", "MTU", str(interface.mtu))
        if client.preshared_key:
            config.set("Peer", "PresharedKey", client.preshared_key)

        config.set("Peer", "PublicKey", interface.public_key)
        config.set("Peer", "AllowedIPs", ", ".join(map(str, interface.allowed_ips)))
        config.set("Peer", "Endpoint", f"{interface.endpoint}:{interface.listen_port}")
        config.set("Peer", "PersistentKeepalive", str(interface.persistent_keepalive))

        with io.StringIO() as fp:
            config.write(fp)
            client_conf = fp.getvalue()

        console = get_console()
        if self.qr:
            qr = QRCode()
            qr.add_data(client_conf)
            with io.StringIO() as fp:
                qr.print_ascii(out=fp)
                qr_code = fp.getvalue()
            console.print(Panel(qr_code, title="Client QR", style="black on white"))
        else:
            console.print(client_conf)

        return 0


class ClientRemoveParser(ClientBaseParser):
    aliases: str = Argument(
        "aliases",
        default=[],
        nargs=argclass.Nargs.ONE_OR_MORE,
        help="Clients alias to remove",
    )

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        for alias in self.aliases:
            try:
                client = Client.load(conn, alias, self.interface)
            except LookupError:
                logging.error(
                    "Error: Client '%s' not found in interface '%s'",
                    alias,
                    self.interface,
                )
                continue
            logging.info("Removing client '%s'", alias)
            client.remove(conn)
        return 0


class ClientListParser(BaseParser):
    """List all clients for an interface"""

    def __call__(self, conn: sqlite3.Connection) -> int:  # type: ignore[override]
        table = SimpleTable(
            "Interface",
            "Client",
            "IPv4",
            "IPv6",
            "Public Key",
            title="WireGuard Clients",
        )

        for interface in Interface.list(conn):
            for client in interface.clients(conn):
                table.add_row(
                    interface.name,
                    client.alias,
                    str(client.ipv4) if client.ipv4 else "",
                    str(client.ipv6) if client.ipv6 else "",
                    client.public_key,
                )

        table.print(self.__parent__.__parent__.output_format)  # type: ignore[union-attr]
        return 0


class ClientCommands(BaseParser):
    """Manage clients for WireGuard interfaces"""

    add: ClientAddParser = ClientAddParser()
    remove: ClientRemoveParser = ClientRemoveParser()
    list: ClientListParser = ClientListParser()

    def __call__(self, *args, **kwargs):
        self.print_help()
        exit(errno.EINVAL)



import configparser
import logging
import os
from pathlib import Path

import rich.logging

from .cli import Parser
from .db import db_connection, init_db


def main(*args):
    xdg_path = Path("~/.local/share/wg-gen").expanduser()
    config_path = xdg_path / "config.ini"
    parser = Parser(
        config_files=[os.getenv("WG_GEN_CONFIG", config_path)],
        auto_env_var_prefix="WG_GEN_",
    )
    parser.parse_args(args or None)

    logging.basicConfig(
        level=parser.log_level,
        handlers=[rich.logging.RichHandler(rich_tracebacks=True, show_time=False)],
        format="%(message)s",
    )

    if parser.db_path is None:
        db_path = xdg_path / "database.sqlite3"
        config = configparser.ConfigParser()

        config.set("DEFAULT", "db_path", str(db_path))
        xdg_path.mkdir(parents=True, exist_ok=True)
        with config_path.open("w") as config_file:
            config.write(config_file)
        parser.db_path = db_path

    with db_connection(parser.db_path) as conn:
        init_db(conn)
        retcode = parser(conn)

    exit(retcode)


if __name__ == "__main__":
    main()



import csv
import json
import sys

import rich
from rich.table import Table


COLORS = [
    "cyan",
    "magenta",
    "yellow",
    "bright_blue",
    "red",
    "green",
    "bright_magenta",
    "dodger_blue2",
    "bright_red",
]


class SimpleTable(Table):
    def __init__(self, *headers, title: str, **kwargs):
        super().__init__(title=title, box=None, **kwargs)

        for idx, header in enumerate(headers):
            self.add_column(header, style=COLORS[idx % len(COLORS)])

    def print_table(self):
        rich.print(self)

    def get_rows(self) -> list[dict[str, str | list[str]]]:
        data: dict[str, list[str | list[str]]] = {}
        cells_len = 0
        for col in self.columns:
            hdr = str(col.header).lower().replace(" ", "_")
            data[hdr] = [
                str(c) if "\n" not in str(c) else str(c).splitlines() for c in col.cells
            ]
            cells_len = len(data[hdr])
        result: list[dict[str, str | list[str]]] = []
        for i in range(cells_len):
            entry: dict[str, str | list[str]] = {}
            for hdr in data:
                entry[hdr] = data[hdr][i]
            result.append(entry)
        return result

    def print_json(self):
        print(json.dumps(self.get_rows(), indent=1))

    def print_csv(self, delimiter: str = ","):
        writer = csv.writer(
            sys.stdout, delimiter=delimiter, quotechar='"', quoting=csv.QUOTE_MINIMAL
        )
        data = self.get_rows()
        if not data:
            return

        writer.writerow(data[0].keys())
        for row in data:
            writer.writerow(
                v if not isinstance(v, list) else ",".join(v) for v in row.values()
            )

    def print(self, format: str = "table"):
        if format == "json":
            self.print_json()
        elif format == "csv":
            self.print_csv()
        elif format == "tsv":
            self.print_csv(delimiter="\t")
        else:
            self.print_table()



import base64

from cryptography.hazmat.primitives import serialization
from cryptography.hazmat.primitives.asymmetric.x25519 import X25519PrivateKey


def keygen():
    priv = X25519PrivateKey.generate()
    priv_b64 = base64.b64encode(
        priv.private_bytes(
            serialization.Encoding.Raw,
            serialization.PrivateFormat.Raw,
            serialization.NoEncryption(),
        ),
    ).decode()
    pub_b64 = base64.b64encode(
        priv.public_key().public_bytes(
            serialization.Encoding.Raw,
            serialization.PublicFormat.Raw,
        ),
    ).decode()
    return priv_b64, pub_b64


def preshared_keygen():
    return base64.b64encode(
        X25519PrivateKey.generate()
        .public_key()
        .public_bytes(
            serialization.Encoding.Raw,
            serialization.PublicFormat.Raw,
        ),
    ).decode()
